@@ -1,6 +1,7 @@
 #ifndef _CACHE_H
 #define _CACHE_H
 
+#include <stdbool.h>
 #include "config.h"
 #include "global.h"
 
@@ -23,7 +24,7 @@ typedef struct type_cached_object {
 void init_cache_module(void);
 t_cached_object *add_to_cache(t_session *session, char *file);
 t_cached_object *search_cache(t_session *session, char *file);
-void done_with_cached_object(t_cached_object *object);
+void done_with_cached_object(t_cached_object *object, bool remove_object);
 void check_cache(void);
 #ifdef HAVE_COMMAND
 void print_cache_list(FILE *fp);
