@@ -5,12 +5,6 @@
 #include "liblist.h"
 #include "session.h"
 
-#define cgi_TIMEOUT              -3
-#define cgi_FORCE_QUIT           -2
-#define cgi_ERROR                -1
-#define cgi_OKE                   0
-#define cgi_END_OF_DATA           1
-
 #define FCGI_VERSION_1           1
 
 #define FCGI_BEGIN_REQUEST       1
@@ -26,13 +20,10 @@
 #define FCGI_UNKNOWN_TYPE       11
 #define FCGI_MAXTYPE (FCGI_UNKNOWN_TYPE)
 
-#define FCGI_REQUEST_COMPLETE    0
-#define FCGI_CANT_MPX_CONN       1
-#define FCGI_OVERLOADED          2
-#define FCGI_UNKNOWN_ROLE        3
-
 #define FCGI_HEADER_LENGTH       8
 
+typedef enum { cgi_TIMEOUT = -3, cgi_FORCE_QUIT, cgi_ERROR, cgi_OKE, cgi_END_OF_DATA } t_cgi_result;
+
 typedef struct type_cgi_info {
 	int timer;
 	struct timeval select_timeout;
@@ -54,27 +45,22 @@ typedef struct type_cgi_info {
 	int fcgi_data_len;
 } t_cgi_info;
 
-typedef struct type_fcgi_program {
-	char  *filename;
-	pid_t pid;
-	int   to_cgi, from_cgi, cgi_error;
-	int   runs, timer;
-	volatile int in_use;
-} t_fcgi_program;
-
-// FastCGI server loadbalancer
+/* FastCGI server loadbalancer
+ */
 void init_load_balancer(t_fcgi_server *fcgi_server);
 t_connect_to *select_connect_to(t_fcgi_server *fcgi_server, t_ipv4 client_ip);
 t_fcgi_server *fcgi_server_match(t_fcgi_server *fcgi_server, t_charlist *fastcgi, char *cgi_script);
 void check_load_balancer(t_config *config);
 
-// Normal CGI
+/* Normal CGI
+ */
 pid_t fork_cgi_process(t_session *session, t_cgi_info *cgi_info);
-int read_from_cgi_process(t_session *session, t_cgi_info *cgi_info);
+t_cgi_result read_from_cgi_process(t_session *session, t_cgi_info *cgi_info);
 
-// FastCGI server
+/* FastCGI server
+ */
 int connect_to_fcgi_server(t_connect_to *connect_to);
 int send_fcgi_request(t_session *session, int sock);
-int read_from_fcgi_server(t_session *session, t_cgi_info *cgi_info);
+t_cgi_result read_from_fcgi_server(t_session *session, t_cgi_info *cgi_info);
 
 #endif
