@@ -116,11 +116,11 @@ void check_delayed_remove_timers(t_config *config) {
 			next = client->next;
 			switch (client->delayed_remove) {
 				case TIMER_OFF:
+					prev = client;
 					break;
 				case 0:
 					free(client->session);
 					free(client);
-					client = prev;
 					if (prev == NULL) {
 						client_list[i] = next;
 					} else {
@@ -129,8 +129,8 @@ void check_delayed_remove_timers(t_config *config) {
 					break;
 				default:
 					client->delayed_remove--;
+					prev = client;
 			}
-			prev = client;
 			client = next;
 		}
 
@@ -409,6 +409,7 @@ void check_ban_list(t_config *config) {
 		next = ban->next;
 		switch (ban->timer) {
 			case TIMER_OFF:
+				prev = ban;
 				break;
 			case 0:
 				if (prev == NULL) {
@@ -418,12 +419,11 @@ void check_ban_list(t_config *config) {
 				}
 				log_unban(config->system_logfile, ban->ip, ban->connect_attempts);
 				free(ban);
-				ban = prev;
 				break;
 			default:
 				ban->timer--;
+				prev = ban;
 		}
-		prev = ban;
 		ban = next;
 	}
 	
@@ -478,6 +478,13 @@ void print_client_list(FILE *fp) {
 		client = client_list[i];
 		while (client != NULL) {
 			fprintf(fp, "  Client ID   : %d\n", client->session->client_id);
+#ifdef DEBUG
+			if (client->session->status != NULL) {
+				fprintf(fp, "  Status      : %s\n", client->session->status);
+			} else {
+				fprintf(fp, "  Status      : unknown\n");
+			}
+#endif
 			ip_address = (unsigned char*)&(client->session->ip_address);
 			fprintf(fp, "  IP-address  : %hhu.%hhu.%hhu.%hhu\n", ip_address[0], ip_address[1], ip_address[2], ip_address[3]);
 			fprintf(fp, "  Socket      : %d\n", client->session->client_socket);
