@@ -94,6 +94,9 @@ void set_environment(t_session *session, t_fcgi_buffer *fcgi_buffer) {
 #ifdef HAVE_SSL
 	char subject[SSL_VAR_SIZE + 1], issuer[SSL_VAR_SIZE + 1];
 #endif
+#ifdef CYGWIN
+	char *root;
+#endif
 
 	add_to_environment(fcgi_buffer, "GATEWAY_INTERFACE", "CGI/1.1");
 
@@ -114,6 +117,14 @@ void set_environment(t_session *session, t_fcgi_buffer *fcgi_buffer) {
 	}
 
 	add_to_environment_chroot(session, fcgi_buffer, "SCRIPT_FILENAME", session->file_on_disk);
+#ifdef CYGWIN
+	if (session->config->platform == windows) {
+		if ((root = strdup(session->host->website_root)) != NULL) {
+			add_to_environment_chroot(session, fcgi_buffer, "DOCUMENT_ROOT", cygwin_to_windows(root));
+			free(root);
+		}
+	} else
+#endif
 	add_to_environment_chroot(session, fcgi_buffer, "DOCUMENT_ROOT", session->host->website_root);
 
 	ip_address = (unsigned char*)&(session->ip_address);
@@ -167,6 +178,10 @@ void set_environment(t_session *session, t_fcgi_buffer *fcgi_buffer) {
 	headerfield_to_environment(session, fcgi_buffer, "Via:", "HTTP_VIA");
 	headerfield_to_environment(session, fcgi_buffer, "X-Forwarded-For:", "HTTP_X_FORWARDED_FOR");
 
+	if (session->path_info != NULL) {
+		add_to_environment(fcgi_buffer, "PATH_INFO", session->path_info);
+	}
+
 	snprintf(value, 9, "%d", session->return_code);
 	add_to_environment(fcgi_buffer, "HTTP_RETURN_CODE", value);
 	if (session->error_code != -1) {
