@@ -162,33 +162,57 @@ void check_target_is_cgi(t_session *session) {
 
 /* Handle an HTTP error.
  */
-int handle_error(t_session *session) {
-	int retval = 200, length;
-	
+int handle_error(t_session *session, int error_code) {
+	char **error_handler, *no_handler = NULL;
+	int length;
+
+	switch (error_code) {
+		case 401:
+			error_handler = &(session->host->error_401_handler);
+			break;
+		case 403:
+			error_handler = &(session->host->error_403_handler);
+			break;
+		case 404:
+			error_handler = &(session->host->error_404_handler);
+			break;
+		case 501:
+			error_handler = &(session->host->error_501_handler);
+			break;
+		case 503:
+			error_handler = &(session->host->error_503_handler);
+			break;
+		default:
+			error_handler = &no_handler;
+	}
+	if (*error_handler == NULL) {
+		return error_code;
+	}
+
+	session->return_code = error_code;
+	session->error_code = error_code;
 	session->handling_error = true;
 	session->mimetype = NULL;
 	if (session->file_on_disk != NULL) {
 		free(session->file_on_disk);
 	}
 	length = strlen(session->host->website_root);
-	if ((session->file_on_disk = (char*)malloc(length + strlen(session->host->error_handler) + 4)) == NULL) { // + 3 for .gz (gzip encoding)
+	if ((session->file_on_disk = (char*)malloc(length + strlen(*error_handler) + 4)) == NULL) { // + 3 for .gz (gzip encoding)
 		return 500;
 	}
 	strcpy(session->file_on_disk, session->host->website_root);
-	strcpy(session->file_on_disk + length, session->host->error_handler);
+	strcpy(session->file_on_disk + length, *error_handler);
 
+	session->cgi_type = none;
 	check_target_is_cgi(session);
 
 	if (session->cgi_type != none) {
-		if (session->host->return_code != -1) {
-			session->return_code = session->host->return_code;
-		}
-		retval = execute_cgi(session);
-	} else {
-		retval = send_file(session);
+		return execute_cgi(session);
+	} else if (is_directory(session->file_on_disk) == no) {
+		return send_file(session);
 	}
 
-	return retval;
+	return error_code;
 }
 
 /* Handle the request from a client.
@@ -240,6 +264,18 @@ int handle_request(t_session *session) {
 		retval = 501;
 	} else if (strcmp(session->method, "PROPFIND") == 0) {
 		retval = 501;
+	} else if (strcmp(session->method, "PROPPATCH") == 0) {
+		retval = 501;
+	} else if (strcmp(session->method, "MKCOL") == 0) {
+		retval = 501;
+	} else if (strcmp(session->method, "COPY") == 0) {
+		retval = 501;
+	} else if (strcmp(session->method, "MOVE") == 0) {
+		retval = 501;
+	} else if (strcmp(session->method, "LOCK") == 0) {
+		retval = 501;
+	} else if (strcmp(session->method, "UNLOCK") == 0) {
+		retval = 501;
 	} else {
 		retval = 400;
 	}
@@ -265,6 +301,9 @@ int fetch_request(t_session *session) {
 		timer = session->binding->time_for_request;
 	}
 
+#ifdef DEBUG
+	set_client_status(session, "fetch_request()");
+#endif
 	do {
 		/* Check if requestbuffer contains a complete request.
 		 */
@@ -317,6 +356,9 @@ int fetch_request(t_session *session) {
 			}
 		}
 
+#ifdef DEBUG
+		set_client_status(session, "fetch_request()::select()");
+#endif
 		FD_ZERO(&read_fds);
 		FD_SET(session->client_socket, &read_fds);
 		switch (select(session->client_socket + 1, &read_fds, NULL, NULL, &select_timeout)) {
@@ -352,6 +394,9 @@ int fetch_request(t_session *session) {
 
 				/* Read from socket.
 				 */
+#ifdef DEBUG
+				set_client_status(session, "fetch_request()::recv()");
+#endif
 #ifdef HAVE_SSL
 				if (session->binding->use_ssl) {
 					bytes_read = ssl_receive(session->ssl_data, session->request + session->bytes_in_buffer, 
@@ -389,6 +434,10 @@ int fetch_request(t_session *session) {
 	increment_transfer(TRANSFER_RECEIVED, session->header_length + session->content_length);
 #endif
 
+#ifdef DEBUG
+	set_client_status(session, NULL);
+#endif
+
 	return result;
 }
 
@@ -524,7 +573,11 @@ int uri_to_path(t_session *session) {
 					if ((retval = get_homedir(session, session->local_user + 1)) != 200) {
 						return retval;
 					}
-					session->host->error_handler = NULL;
+					session->host->error_401_handler = NULL;
+					session->host->error_403_handler = NULL;
+					session->host->error_404_handler = NULL;
+					session->host->error_501_handler = NULL;
+					session->host->error_503_handler = NULL;
 				} else {
 					return 500;
 				}
@@ -542,6 +595,7 @@ int uri_to_path(t_session *session) {
 		alias_length = strlen(alias->key);
 		if (strncmp(session->uri, alias->key, alias_length) == 0) {
 			if ((*(session->uri + alias_length) == '/') || (*(session->uri + alias_length) == '\0')) {
+				session->alias_used = true;
 				break;
 			}
 		}
@@ -597,6 +651,51 @@ t_access allow_client(t_session *session) {
 	return unknown;
 }
 
+int get_path_info(t_session *session) {
+	char *uri, *dot, *slash;
+	int ofs = 0;
+	int len;
+
+	if (session->alias_used) {
+		return 0;
+	}
+
+	len = strlen(session->host->website_root);
+	if (len >= strlen(session->file_on_disk)) {
+		return -1;
+	}
+
+	uri = session->file_on_disk + len;
+	if ((dot = strchr(uri, '.')) == NULL) {
+		return 0;
+	}
+
+	if ((slash = strchr(dot, '/')) == NULL) {
+		return 0;
+	}
+
+	*slash = '\0';
+	check_target_is_cgi(session);
+	*slash = '/';
+	if (session->cgi_type != none) {
+		if ((session->path_info = strdup(slash)) == NULL) {
+			return -1;
+		}
+
+		ofs = slash - uri;
+		if (ofs < strlen(session->uri)) {
+			*(session->uri + ofs) = '\0';
+		} else {
+			return -1;
+		}
+		*slash = '\0';
+
+		return 1;
+	}
+
+	return 0;
+}
+
 /* Serve the client that connected to the webserver
  */
 int serve_client(t_session *session) {
@@ -699,6 +798,11 @@ int serve_client(t_session *session) {
 			session->uri_is_dir = true;
 			break;
 		case no:
+			if ((put_method == false) && (session->host->enable_path_info)) {
+				if (get_path_info(session) == -1) {
+					return 500;
+				}
+			}
 			break;
 		case no_access:
 			log_error(session, fb_filesystem);
@@ -711,7 +815,8 @@ int serve_client(t_session *session) {
 					}
 				}
 			}
-			if ((session->accept_gzip == false) && (put_method == false)) {
+
+			if ((put_method == false) && (session->accept_gzip == false)) {
 				return 404;
 			}
 	}
@@ -805,11 +910,6 @@ void handle_request_result(t_session *session, int result) {
 		case 200:
 			break;
 		case 204:
-			if (session->data_sent == false) {
-				send_header(session, 204);
-				send_buffer(session, "\r\n", 2);
-			}
-			break;
 		case 304:
 		case 412:
 		case 413:
@@ -821,38 +921,47 @@ void handle_request_result(t_session *session, int result) {
 		case 400:
 			log_garbage(session);
 			if (session->data_sent == false) {
-				send_code(session, result);
+				if (send_code(session, result) == -1) {
+					session->keep_alive = false;
+				}
 			}
 			if ((session->config->ban_on_garbage > 0) && ip_allowed(session->ip_address, session->config->banlist_mask)) {
 				ban_ip(session->ip_address, session->config->ban_on_garbage, session->config->kick_on_ban);
 				log_system(session, "Client banned because of sending garbage");
 			}
 			break;
+		case 401:
+		case 403:
+		case 404:
+		case 501:
+		case 503:
+			if (session->data_sent == false) {
+				result = handle_error(session, result);
+				switch (result) {
+					case -1:
+						session->keep_alive = false;
+						break;
+					case 200:
+						break;
+					default:
+						if (result == 500) {
+							session->keep_alive = false;
+						}
+						if (session->data_sent == false) {
+							if (send_code(session, result) == -1) {
+								session->keep_alive = false;
+							}
+						}
+				}
+			}
+			break;
 		case 500:
 			session->keep_alive = false;
 		default:
 			if (session->data_sent == false) {
-				if ((session->host->error_handler == NULL) || (result == 301)) {
-					if (send_code(session, result) == -1) {
-						session->keep_alive = false;
-					}
-				} else {
-					session->error_code = result;
-					switch (handle_error(session)) {
-						case -1:
-							session->keep_alive = false;
-						case 200:
-							break;
-						default:
-							if (send_code(session, result) == -1) {
-								session->keep_alive = false;
-							}
-					}
-				}
+				send_code(session, result);
 			}
 	}
-
-	restore_host(session);
 }
 
 /* Handle the connection of a client.
@@ -1059,9 +1168,6 @@ int accept_connection(t_binding *binding, t_config *config) {
 			log_system(session, "Maximum number of total connections reached");
 			break;
 		case ca_BANNED:
-#ifdef DEBUG
-			log_system(session, "Client kicked because of ban");
-#endif
 			if (config->reban_during_ban && ip_allowed(session->ip_address, session->config->banlist_mask)) {
 				reban_ip(session->ip_address);
 			}
@@ -1126,10 +1232,6 @@ int run_server(t_settings *settings) {
 		fprintf(stderr, "Error while reading mimetype configuration.\n");
 		return EXIT_FAILURE;
 	}
-	if (read_throttleconfig(config->throttle_config, &(config->throttle)) == -1) {
-		fprintf(stderr, "Error while reading throttle configuration.\n");
-		return EXIT_FAILURE;
-	}
 
 	if (settings->config_check) {
 		printf("Configuration oke.\n");
@@ -1303,9 +1405,6 @@ int run_server(t_settings *settings) {
 					case rs_NONE:
 						break;
 					case rs_QUIT_SERVER:
-#ifdef DEBUG
-						log_string(config->system_logfile, "Received TERM signal");
-#endif
 						must_quit = true;
 						break;
 					case rs_UNBAN_CLIENTS:
