@@ -399,8 +399,7 @@ const char *http_error(int code) {
 /*
 		{504, "Gateway Timeout"},
 */
-		{505, "HTTP Version Not Supported"},
-		{0  , "Unknown Error"}
+		{505, "HTTP Version Not Supported"}
 	};
 
 	for (i=0; error[i].code != 0; i++) {
