@@ -12,7 +12,8 @@ char *strlower(char *line);
 int split_string(const char *str, char **key, char **value, char c);
 int split_configline(const char *str, char **key, char **value);
 bool valid_uri(char *uri);
-int parse_special_chars(char *line, int length);
+void url_decode(char *line);
+void remove_dangerous_chars(char *line);
 int prevent_CMDi(char *line);
 int prevent_SQLi(char *line, int length, char **newline);
 int prevent_XSS(char *line);
