@@ -130,6 +130,10 @@ void log_request(t_session *session, int code) {
 			snprintf(str + offset, BUFFER_SIZE - offset, "%d|%lld|%s|%s %s", code, (long long)session->bytes_sent, secure_string(user), secure_string(session->method), secure_string(session->uri));
 			offset += strlen(str + offset);
 
+			if ((offset < BUFFER_SIZE) && (session->path_info != NULL)) {
+				snprintf(str + offset, BUFFER_SIZE - offset, "/%s", secure_string(session->path_info));
+				offset += strlen(str + offset);
+			}
 			if ((offset < BUFFER_SIZE) && (session->vars != NULL)) {
 				snprintf(str + offset, BUFFER_SIZE - offset, "?%s", secure_string(session->vars));
 				offset += strlen(str + offset);
