@@ -39,7 +39,7 @@ char *hs_range  = "Accept-Ranges: bytes\r\n";   // 22
 char *hs_gzip   = "Content-Encoding: gzip\r\n"; // 24
 char *hs_eol    = "\r\n";                       //  2
 
-char *unknown_code = "unknown error code";
+char *unknown_code = "Unknown Error";
 
 /* Send a char buffer to the client. Traffic throttling is handled here.
  */
@@ -80,6 +80,10 @@ static int send_to_client(t_session *session, const char *buffer, int size) {
 			can_send = rest;
 		}
 		
+#ifdef DEBUG
+	    set_client_status(session, "send_to_client()::send()");
+#endif
+
 #ifdef HAVE_SSL
 		if (session->binding->use_ssl) {
 			if ((bytes_sent = ssl_send(session->ssl_data, (char*)(buffer + total_sent), can_send)) <= 0) {
@@ -90,6 +94,9 @@ static int send_to_client(t_session *session, const char *buffer, int size) {
 			if ((bytes_sent = send(session->client_socket, (char*)(buffer + total_sent), can_send, 0)) <= 0) {
 				bytes_sent = -1;
 			}
+#ifdef DEBUG
+	    set_client_status(session, NULL);
+#endif
 
 		if (bytes_sent == -1) {
 			if (errno != EINTR) {
