@@ -37,8 +37,11 @@ static t_host *new_host(void) {
 	if ((host = (t_host*)malloc(sizeof(t_host))) != NULL) {
 		host->website_root       = NULL;
 		host->start_file         = "index.html";
-		host->error_handler      = NULL;
-		host->return_code        = -1;
+		host->error_401_handler  = NULL;
+		host->error_403_handler  = NULL;
+		host->error_404_handler  = NULL;
+		host->error_501_handler  = NULL;
+		host->error_503_handler  = NULL;
 		host->access_logfile     = LOG_DIR"/access.log";
 		host->access_fileptr     = NULL;
 		host->access_fp          = &(host->access_fileptr);
@@ -75,6 +78,7 @@ static t_host *new_host(void) {
 		host->prevent_sqli       = false;
 		host->prevent_xss        = false;
 		host->follow_symlinks    = false;
+		host->enable_path_info   = false;
 		init_charlist(&(host->fast_cgi));
 		host->secure_url         = true;
 
@@ -178,7 +182,6 @@ t_config *default_config(void) {
 	t_config *config;
 
 	if ((config = (t_config*)malloc(sizeof(t_config))) != NULL) {
-		config->throttle_config    = "throttle.conf";
 		config->mimetype_config    = "mimetype.conf";
 
 		config->binding            = NULL;
@@ -234,6 +237,9 @@ t_config *default_config(void) {
 		config->reconnect_delay    = 0;
 		config->banlist_mask       = NULL;
 		config->fcgi_server        = NULL;
+#ifdef CYGWIN
+		config->platform           = windows;
+#endif
 
 #ifdef HAVE_CACHE
 		config->cache_size         = 10485760;  //  10MB
@@ -503,6 +509,8 @@ int check_configuration(t_config *config) {
 static bool system_setting(char *key, char *value, t_config *config) {
 	char *uid, *gid, *rest;
 	t_cgi_handler *CGI;
+	t_throttle *throt;
+	int speed;
 #ifdef HAVE_COMMAND
 	t_binding *binding;
 	char *port, *password;
@@ -510,8 +518,17 @@ static bool system_setting(char *key, char *value, t_config *config) {
 
 #ifdef HAVE_SSL
 	if (strcmp(key, "allowedciphers") == 0) {
-		if ((config->allowed_ciphers = strdup(value)) != NULL) {
-			return true;
+		if (config->allowed_ciphers == NULL) {
+			if ((config->allowed_ciphers = strdup(value)) != NULL) {
+				return true;
+			}
+		} else {
+			if ((rest = (char*)malloc(strlen(config->allowed_ciphers) + strlen(value) + 2)) != NULL) {
+				sprintf(rest, "%s:%s", config->allowed_ciphers, value);
+				free(config->allowed_ciphers);
+				config->allowed_ciphers = rest;
+				return true;
+			}
 		}
 	} else
 #endif
@@ -663,6 +680,16 @@ static bool system_setting(char *key, char *value, t_config *config) {
 				return true;
 			}
 		}
+#ifdef CYGWIN
+	} else if (strcmp(key, "platform") == 0) {
+		if (strcmp(value, "windows") == 0) {
+			config->platform = windows;
+			return true;
+		} else if (strcmp(value, "cygwin") == 0) {
+			config->platform = cygwin;
+			return true;
+		}
+#endif
 	} else if (strcmp(key, "reconnectdelay") == 0) {
 		if ((config->reconnect_delay = str2int(value)) != -1) {
 			return true;
@@ -718,9 +745,31 @@ static bool system_setting(char *key, char *value, t_config *config) {
 				return true;
 			}
 		}
-	} else if (strcmp(key, "throttleconfig") == 0) {
-		if ((config->throttle_config = strdup(value)) != NULL) {
-			return true;
+	} else if (strcmp(key, "throttle") == 0) {
+		if (split_string(value, &rest, &value, ':') != -1) {
+			if (((*rest == '.') || (strchr(rest, '/') != NULL)) && (speed = str2int(value)) > 0) {
+				if (config->throttle == NULL) {
+					if ((config->throttle = (t_throttle*)malloc(sizeof(t_throttle))) == NULL) {
+						return false;
+					}
+					throt = config->throttle;
+				} else {
+					throt = config->throttle;
+					while (throt->next != NULL) {
+						throt = throt->next;
+					}
+					if ((throt->next = (t_throttle*)malloc(sizeof(t_throttle))) == NULL) {
+						return false;
+					}
+					throt = throt->next;
+				}
+				throt->next = NULL;
+
+				if ((throt->filetype = strlower(strdup(rest))) != NULL) {
+					throt->upload_speed = speed << 10; // convert to kilobytes
+					return true;
+				}
+			}
 		}
 	} else if (strcmp(key, "timeforcgi") == 0) {
 		if ((config->time_for_cgi = str2int(value)) >= 1) {
@@ -743,8 +792,8 @@ static bool system_setting(char *key, char *value, t_config *config) {
 }
 
 static bool user_setting(char *key, char *value, t_host *host, t_tempdata **tempdata) {
-	char *code;
-	int result;
+	char *code, **handler, *pwd = NULL, *grp = NULL;
+	t_keyvalue *kv;
 
 	if (strcmp(key, "accesslist") == 0) {
 		if ((host->access_list = parse_accesslist(value, true, host->access_list)) != NULL) {
@@ -752,7 +801,6 @@ static bool user_setting(char *key, char *value, t_host *host, t_tempdata **temp
 		}
 	} else if (strcmp(key, "altergroup") == 0) {
 		if (parse_charlist(value, &(host->alter_group)) != -1) {
-			register_tempdata(tempdata, &(host->alter_group), tc_CHARLIST);
 			return true;
 		}
 	} else if (strcmp(key, "alterlist") == 0) {
@@ -764,43 +812,80 @@ static bool user_setting(char *key, char *value, t_host *host, t_tempdata **temp
 			return true;
 		}
 	} else if (strcmp(key, "errorhandler") == 0) {
-		result = split_string(value, &value, &code, ',');
-		if ((strlen(value) < 128) && (*value == '/')) {
-			if ((host->error_handler = strdup(value)) != NULL) {
-				if (result == 0) {
-					if ((host->return_code = str2int(code)) != -1) {
-						return true;
+		if (split_string(value, &code, &value, ':') == 0) {
+			switch (str2int(code)) {
+				case 401:
+					handler = &(host->error_401_handler);
+					break;
+				case 403:
+					handler = &(host->error_403_handler);
+					break;
+				case 404:
+					handler = &(host->error_404_handler);
+					break;
+				case 501:
+					handler = &(host->error_501_handler);
+					break;
+				case 503:
+					handler = &(host->error_503_handler);
+					break;
+				default:
+					return false;
+			}
+			if ((strlen(value) < 128) && (*value == '/')) {
+				if ((*handler = strdup(value)) != NULL) {
+					if (register_tempdata(tempdata, *handler, tc_data) == -1) {
+						free(*handler);
+						*handler = NULL;
+						return false;
 					}
-				} else {
 					return true;
 				}
 			}
 		}
 	} else if (strcmp(key, "indexstyle") == 0) {
 		if ((host->index_style = strdup(value)) != NULL) {
-			register_tempdata(tempdata, host->index_style, tc_DATA);
-			return true;
+			if (register_tempdata(tempdata, host->index_style, tc_data) != -1) {
+				return true;
+			} else {
+				free(host->index_style);
+				host->index_style = NULL;
+			}
 		}
 	} else if (strcmp(key, "loginmessage") == 0) {
 		if (strlen(value) < 64) {
 			if ((host->login_message = strdup(value)) != NULL) {
-				register_tempdata(tempdata, host->login_message, tc_DATA);
-				return true;
+				if (register_tempdata(tempdata, host->login_message, tc_data) != -1) {
+					return true;
+				} else {
+					free(host->login_message);
+					host->login_message = NULL;
+				}
 			}
 		}
 	} else if (strcmp(key, "passwordfile") == 0) {
-		if (parse_credentialfiles(value, &(host->auth_method), &(host->passwordfile), &(host->groupfile)) == 0) {
-			if (host->passwordfile != NULL) {
-				register_tempdata(tempdata, host->passwordfile, tc_DATA);
+		if (parse_credentialfiles(value, &(host->auth_method), &pwd, &grp) == 0) {
+			if (pwd != NULL) {
+				if (register_tempdata(tempdata, pwd, tc_data) == -1) {
+					free(pwd);
+					if (grp != NULL) {
+						free(grp);
+					}
+					return false;
+				}
+				host->passwordfile = pwd;
 			}
-			if (host->groupfile != NULL) {
-				register_tempdata(tempdata, host->groupfile, tc_DATA);
+			if (grp != NULL) {
+				if (register_tempdata(tempdata, grp, tc_data) == -1) {
+					free(grp);
+					return false;
+				}
+				host->groupfile = grp;
 			}
 			return true;
 		}
 	} else if (strcmp(key, "requiredgroup") == 0) {
 		if (parse_charlist(value, &(host->required_group)) != -1) {
-			register_tempdata(tempdata, &(host->required_group), tc_CHARLIST);
 			return true;
 		}
 #ifdef HAVE_SSL
@@ -811,8 +896,15 @@ static bool user_setting(char *key, char *value, t_host *host, t_tempdata **temp
 #endif
 	} else if (strcmp(key, "setenv") == 0) {
 		if (parse_keyvaluelist(value, &(host->envir_str), "=") != -1) {
-			register_tempdata(tempdata, host->envir_str, tc_KEYVALUE);
-			return true;
+			if (register_tempdata(tempdata, host->envir_str, tc_keyvalue) != -1) {
+				return true;
+			} else {
+				kv = host->envir_str;
+				host->envir_str = host->envir_str->next;
+				free(kv->key);
+				free(kv->value);
+				free(kv);
+			}
 		}
 	} else if (strcmp(key, "showindex") == 0) {
 		if (parse_yesno(value, &(host->show_index)) == 0) {
@@ -821,8 +913,12 @@ static bool user_setting(char *key, char *value, t_host *host, t_tempdata **temp
 	} else if (strcmp(key, "startfile") == 0) {
 		if (valid_file(value)) {
 			if ((host->start_file = strdup(value)) != NULL) {
-				register_tempdata(tempdata, host->start_file, tc_DATA);
-				return true;
+				if (register_tempdata(tempdata, host->start_file, tc_data) != -1) {
+					return true;
+				} else {
+					free(host->start_file);
+					host->start_file = NULL;
+				}
 			}
 		}
 	} else if (strcmp(key, "usegzfile") == 0) {
@@ -852,10 +948,6 @@ static bool host_setting(char *key, char *value, t_host *host) {
 				}
 			}
 		}
-	} else if (strcmp(key, "wrapcgi") == 0) {
-		if ((host->wrap_cgi = strdup(value)) != NULL) {
-			return true;
-		}
 	} else if (strcmp(key, "denybot") == 0) {
 		if (split_string(value, &botname, &value, ':') == 0) {
 			if ((denybot = (t_denybotlist*)malloc(sizeof(t_denybotlist))) != NULL) {
@@ -870,6 +962,10 @@ static bool host_setting(char *key, char *value, t_host *host) {
 				}
 			}
 		}
+	} else if (strcmp(key, "enablepathinfo") == 0) {
+		if (parse_yesno(value, &(host->enable_path_info)) == 0) {
+			return true;
+		}
 	} else if (strcmp(key, "errorlogfile") == 0) {
 		if (*value == '/') {
 			if ((host->error_logfile = strdup(value)) != NULL) {
@@ -944,6 +1040,10 @@ static bool host_setting(char *key, char *value, t_host *host) {
 				return true;
 			}
 		}
+	} else if (strcmp(key, "wrapcgi") == 0) {
+		if ((host->wrap_cgi = strdup(value)) != NULL) {
+			return true;
+		}
 	}
 	
 	return false;
@@ -1133,12 +1233,7 @@ static bool fcgi_server_setting(char *key, char *value, t_fcgi_server *fcgi_serv
 	char *port, *rest;
 	t_connect_to *connect_to;
 
-	if (strcmp(key, "serverroot") == 0) {
-		if ((fcgi_server->chroot = strdup(value)) != NULL) {
-			fcgi_server->chroot_len = strlen(fcgi_server->chroot);
-			return true;
-		}
-	} else if (strcmp(key, "connectto") == 0) {
+	if (strcmp(key, "connectto") == 0) {
 		while (value != NULL) {
 			split_string(value, &value, &rest, ',');
 
@@ -1167,6 +1262,11 @@ static bool fcgi_server_setting(char *key, char *value, t_fcgi_server *fcgi_serv
 		if ((fcgi_server->fcgi_id = strdup(value)) != NULL) {
 			return true;
 		}
+	} else if (strcmp(key, "serverroot") == 0) {
+		if ((fcgi_server->chroot = strdup(value)) != NULL) {
+			fcgi_server->chroot_len = strlen(fcgi_server->chroot);
+			return true;
+		}
 	} else if (strcmp(key, "sessiontimeout") == 0) {
 		if ((fcgi_server->session_timeout = 60 * str2int(value)) >= 0) {
 			return true;
@@ -1452,96 +1552,82 @@ int read_user_configfile(char *configfile, t_host *host, t_tempdata **tempdata)
 	int  retval, counter, lines_read;
 	FILE *fp;
 	char line[MAX_LENGTH_CONFIGLINE + 1], *key, *value;
-	t_accesslist *access, *alter;
+	t_accesslist *acs_list = NULL, *alt_list = NULL;
+	t_charlist req_grp, alt_grp;
 
-	if ((fp = fopen(configfile, "r")) != NULL) {
-		line[MAX_LENGTH_CONFIGLINE] = '\0';
-		counter = retval = 0;
+	if ((fp = fopen(configfile, "r")) == NULL) {
+		return -1;
+	}
 
-		access = host->access_list;
+	line[MAX_LENGTH_CONFIGLINE] = '\0';
+	counter = retval = 0;
+
+	if (tempdata != NULL) {
+		acs_list = host->access_list;
 		host->access_list = NULL;
-		alter = host->alter_list;
+		alt_list = host->alter_list;
 		host->alter_list = NULL;
 
-		while ((lines_read = fgets_multi(line, MAX_LENGTH_CONFIGLINE, fp)) != 0) {
-			if ((lines_read == -1) || (strlen(line) > MAX_LENGTH_CONFIGLINE - 1)) {
-				retval = counter + 1;
-				fprintf(stderr, "Line %d in %s is too long.\n", retval, configfile);
-				break;
-			}
-			counter += lines_read;
+		copy_charlist(&alt_grp, &(host->alter_group));
+		init_charlist(&(host->alter_group));
+		copy_charlist(&req_grp, &(host->required_group));
+		init_charlist(&(host->required_group));
+	}
 
-			key = strip_line(line);
-			if (*key != '\0') {
-				if (split_configline(key, &key, &value) != -1) {
-					strlower(key);
-					if (user_setting(key, value, host, tempdata) == false) {
-						retval = counter;
-						break;
-					}
-				} else {
+	while ((lines_read = fgets_multi(line, MAX_LENGTH_CONFIGLINE, fp)) != 0) {
+		if ((lines_read == -1) || (strlen(line) > MAX_LENGTH_CONFIGLINE - 1)) {
+			retval = counter + 1;
+			fprintf(stderr, "Line %d in %s is too long.\n", retval, configfile);
+			break;
+		}
+		counter += lines_read;
+
+		key = strip_line(line);
+		if (*key != '\0') {
+			if (split_configline(key, &key, &value) != -1) {
+				strlower(key);
+				if (user_setting(key, value, host, tempdata) == false) {
 					retval = counter;
 					break;
 				}
-			}
-		}
-		fclose(fp);
-
-		if (host->access_list == NULL) {
-			host->access_list = access;
-		} else {
-			register_tempdata(tempdata, host->access_list, tc_ACCESSLIST);
-		}
-		if (host->alter_list == NULL) {
-			host->alter_list = alter;
-		} else {
-			register_tempdata(tempdata, host->alter_list, tc_ACCESSLIST);
-		}
-
-		return retval;
-	}
-
-	return -1;
-}
-
-int read_throttleconfig(char *configfile, t_throttle **throttle) {
-	FILE *fp;
-	char line[MAX_LENGTH_CONFIGLINE + 1], *key, *value;
-	t_throttle *throt = NULL;
-	int speed;
-
-	*throttle = NULL;
-	if ((fp = fopen(configfile, "r")) == NULL) {
-		return 1;
-	}
-	line[MAX_LENGTH_CONFIGLINE] = '\0';
-
-	while (fgets(line, MAX_LENGTH_CONFIGLINE, fp) != NULL) {
-		if ((*line != '#') && (strlen(line) > 0)) {
-			if (split_string(line, &key, &value, ':') != -1) {
-				if (((*key == '.') || (strchr(key, '/') != NULL)) && (speed = str2int(value)) > 0) {
-					if (throt == NULL) {
-						if ((*throttle = throt = (t_throttle*)malloc(sizeof(t_throttle))) == NULL) {
-							return -1;
-						}
-					} else {
-						if ((throt->next = (t_throttle*)malloc(sizeof(t_throttle))) == NULL) {
-							return -1;
-						}
-						throt = throt->next;
-					}
-					throt->next = NULL;
-					if ((throt->filetype = strlower(strdup(key))) == NULL) {
-						return -1;
-					}
-					throt->upload_speed = speed << 10; // convert to kilobytes
-				}
+			} else {
+				retval = counter;
+				break;
 			}
 		}
 	}
+
 	fclose(fp);
 
-	return 0;
+	if (tempdata != NULL) {
+		if (host->access_list == NULL) {
+			host->access_list = acs_list;
+		} else if (register_tempdata(tempdata, host->access_list, tc_accesslist) == -1) {
+			host->access_list = remove_accesslist(host->access_list);
+			retval = -1;
+		}
+		if (host->alter_list == NULL) {
+			host->alter_list = alt_list;
+		} else if (register_tempdata(tempdata, host->alter_list, tc_accesslist) == -1) {
+			host->alter_list = remove_accesslist(host->alter_list);
+			retval = -1;
+		}
+
+		if (host->alter_group.size == 0) {
+			copy_charlist(&(host->alter_group), &alt_grp);
+		} else if (register_tempdata(tempdata, &(host->alter_group), tc_charlist) == -1) {
+			remove_charlist(&(host->alter_group));
+			retval = -1;
+		}
+		if (host->required_group.size == 0) {
+			copy_charlist(&(host->required_group), &req_grp);
+		} else if (register_tempdata(tempdata, &(host->required_group), tc_charlist) == -1) {
+			remove_charlist(&(host->required_group));
+			retval = -1;
+		}
+	}
+
+	return retval;
 }
 
 /******************************************************************************
