@@ -16,6 +16,9 @@
 typedef enum { root, part } t_pathmatch;
 typedef enum { basic, digest } t_auth_method;
 typedef enum { hiawatha, common, extended } t_log_format;
+#ifdef CYGWIN
+typedef enum { windows, cygwin } t_platform;
+#endif
 
 typedef struct type_cgi_handler {
 	char          *handler;
@@ -32,7 +35,7 @@ typedef struct type_connect_to {
 } t_connect_to;
 
 typedef struct type_cgi_session {
-	t_ipv4 client_ip;
+	t_ipv4        client_ip;
 	t_connect_to  *connect_to;
 	int timer;
 
@@ -124,8 +127,11 @@ typedef struct type_directory {
 typedef struct type_host {
 	char          *website_root;
 	char          *start_file;
-	char          *error_handler;
-	int           return_code;
+	char          *error_401_handler;
+	char          *error_403_handler;
+	char          *error_404_handler;
+	char          *error_501_handler;
+	char          *error_503_handler;
 	char          *access_logfile;
 	FILE          *access_fileptr;
 	FILE          **access_fp;
@@ -162,6 +168,7 @@ typedef struct type_host {
 	bool          prevent_sqli;
 	bool          prevent_xss;
 	bool          follow_symlinks;
+	bool          enable_path_info;
 	t_charlist    fast_cgi;
 
 	struct type_host *next;
@@ -170,7 +177,6 @@ typedef struct type_host {
 } t_host;
 
 typedef struct type_config {
-	char          *throttle_config;
 	char          *mimetype_config;
 
 	char          *server_root;
@@ -206,6 +212,9 @@ typedef struct type_config {
 	t_cgi_handler  *cgi_handler;
 	char          *cgi_wrapper;
 	bool          wrap_user_cgi;
+#ifdef CYGWIN
+	t_platform    platform;
+#endif
 
 	int           ban_on_garbage;
 	int           ban_on_max_per_ip;
@@ -238,7 +247,6 @@ int check_configuration(t_config *config);
 int read_main_configfile(char *configfile, t_config *config, bool config_check);
 int read_user_configfile(char *configfile, t_host *host, t_tempdata **tempdata);
 t_host *get_hostrecord(t_host *host, char *hostname, t_binding *binding);
-int read_throttleconfig(char *configfile, t_throttle **throttle);
 unsigned short get_throttlespeed(char *type, t_throttle *throttle);
 
 #ifdef HAVE_SSL
