@@ -12,6 +12,7 @@
 #include <pthread.h>
 #include <pwd.h>
 #include "libstr.h"
+#include "liblist.h"
 #include "session.h"
 #ifdef HAVE_COMMAND
 #include "command.h"
@@ -31,6 +32,8 @@ static void clear_session(t_session *session) {
 	session->uri_is_dir = false;
 	session->accept_gzip = false;
 	session->encode_gzip = false;
+	session->path_info = NULL;
+	session->alias_used = false;
 	session->vars = NULL;
 	session->http_version = NULL;
 	session->headerfields = NULL;
@@ -60,6 +63,9 @@ static void clear_session(t_session *session) {
 	session->return_code = 200;
 	session->error_code = -1;
 	session->tempdata = NULL;
+#ifdef DEBUG
+	session->status = NULL;
+#endif
 }
 
 /* Initialize a session-record.
@@ -94,6 +100,10 @@ void reset_session(t_session *session) {
 	sfree(session->file_on_disk);
 	sfree(session->local_user);
 	sfree(session->remote_user);
+	sfree(session->path_info);
+#ifdef DEBUG
+	sfree(session->status);
+#endif
 	remove_headerfields(session);
 	if (session->directory != NULL) {
 		pthread_mutex_lock(&(session->directory->client_mutex));
@@ -106,7 +116,6 @@ void reset_session(t_session *session) {
 		}
 		pthread_mutex_unlock(&(session->directory->client_mutex));
 	}
-	remove_tempdata(session->tempdata);
 
 	/* HTTP pipelining
 	 */
@@ -122,6 +131,11 @@ void reset_session(t_session *session) {
 		session->bytes_in_buffer = 0;
 	}
 	
+	remove_tempdata(session->tempdata);
+	if (session->host_copied) {
+		free(session->host);
+	}
+
 	clear_session(session);
 }
 
@@ -213,9 +227,11 @@ void remove_headerfields(t_session *session) {
 int get_homedir(t_session *session, char *username) {
 	struct passwd *pwd;
 	int len;
+	char *old_root;
 
 	if ((pwd = getpwnam(username)) != NULL) {
 		len = strlen(pwd->pw_dir);
+		old_root = session->host->website_root;
 		if ((session->host->website_root = (char*)malloc(len + strlen(session->config->user_directory) + 2)) == NULL) {
 			return -1;
 		}
@@ -223,7 +239,11 @@ int get_homedir(t_session *session, char *username) {
 		memcpy(session->host->website_root, pwd->pw_dir, len);
 		*(session->host->website_root + len) = '/';
 		strcpy(session->host->website_root + len + 1, session->config->user_directory);
-		register_tempdata(&(session->tempdata), session->host->website_root, tc_DATA);
+		if (register_tempdata(&(session->tempdata), session->host->website_root, tc_data) == -1) {
+			free(session->host->website_root);
+			session->host->website_root = old_root;
+			return -1;
+		}
 
 		return 200;
 	}
@@ -251,16 +271,6 @@ bool duplicate_host(t_session *session) {
 	return true;
 }
 
-/* Restore the active host-record.
- */
-void restore_host(t_session *session) {
-	if (session->host_copied) {
-		free(session->host);
-		session->host_copied = false;
-	}
-	session->host = session->config->first_host;
-}
-
 /* Is the requested file marked as volatile?
  */
 bool is_volatile_object(t_session *session) {
@@ -425,3 +435,16 @@ void close_socket(t_session *session) {
 		session->socket_open = false;
 	}
 }
+
+// --< debug >-----------------------------------------------------------------
+
+#ifdef DEBUG
+void set_client_status(t_session *session, char *status) {
+	sfree(session->status);
+	if (status != NULL) {
+		session->status = strdup(status);
+	} else {
+		session->status = NULL;
+	}
+}
+#endif
