@@ -12,6 +12,11 @@
 #define MAX_CHILD_ID 30000
 #define OUTPUT_BUFFER_SIZE 2000
 
+typedef enum { none, program, script, fastcgi} t_cgi_type;
+#ifdef HAVE_SSL
+typedef enum { missing_slash, require_ssl } t_cause_of_301;
+#endif
+
 typedef struct type_headerfield {
 	char *data;
 	int  value_offset;
@@ -19,11 +24,6 @@ typedef struct type_headerfield {
 	struct type_headerfield *next;
 } t_headerfield;
 
-#ifdef HAVE_SSL
-enum t_cause_of_301 { missing_slash, require_ssl };
-#endif
-enum t_cgi_type { none, program, script, fastcgi};
-
 typedef struct type_session {
 	t_config        *config;
 
@@ -34,10 +34,10 @@ typedef struct type_session {
 	bool            head_request;
 	bool            keep_alive;
 	int             kept_alive;
-	enum t_cgi_type cgi_type;
+	t_cgi_type      cgi_type;
 	char            *cgi_handler;
 	t_fcgi_server   *fcgi_server;
-	char            *request, *method, *uri, *vars, *http_version, *body, *file_on_disk;
+	char            *request, *method, *uri, *path_info, *vars, *http_version, *body, *file_on_disk;
 	long            header_length, content_length, buffer_size, bytes_in_buffer;
 	bool            data_sent;
 	char            *local_user;
@@ -45,6 +45,7 @@ typedef struct type_session {
 	bool            uri_is_dir;
 	bool            accept_gzip;
 	bool            encode_gzip;
+	bool            alias_used;
 	t_headerfield   *headerfields;
 	t_ipv4          ip_address;
 	bool            ip_resolved;
@@ -59,7 +60,7 @@ typedef struct type_session {
 	off_t           bytes_sent;
 	int             return_code;
 	int             error_code;
-	t_tempdata     *tempdata;
+	t_tempdata      *tempdata;
 
 	/* Throttling: send_buffer() in send.c
 	 */
@@ -76,13 +77,17 @@ typedef struct type_session {
 	 */
 #ifdef HAVE_SSL
 	SSL             *ssl_data;
-	enum t_cause_of_301 cause_of_301;
+	t_cause_of_301  cause_of_301;
 #endif
 
 	/* Output buffer
 	 */
 	char            output_buffer[OUTPUT_BUFFER_SIZE];
 	int             output_size;
+
+#ifdef DEBUG
+	char            *status;
+#endif
 } t_session;
 
 void init_session(t_session *session);
@@ -96,11 +101,14 @@ void remove_headerfields(t_session *session);
 
 int  get_homedir(t_session *session, char *username);
 bool duplicate_host(t_session *session);
-void restore_host(t_session *session);
 bool is_volatile_object(t_session *session);
 int  copy_directory_settings(t_session *session);
 char *file_in_chroot(t_session *session, char *file);
 bool client_is_rejected_bot(t_session *session);
 void close_socket(t_session *session);
 
+#ifdef DEBUG
+void set_client_status(t_session *session, char *status);
+#endif
+
 #endif
