@@ -72,8 +72,8 @@ int send_file(t_session *session) {
 	char *referer, *buffer = NULL, value[VALUE_SIZE + 1], *dot, *date, *range, *range_begin, *range_end;
 	long bytes_read, total_bytes, size, speed;
 	off_t file_size, send_begin, send_end, send_size;
-	int  retval, handle, http_code;
-	bool invalid_referer = false, prot_oke;
+	int  retval, handle;
+	bool invalid_referer, prot_oke;
 	struct stat status;
 	struct tm *fdate;
 #ifdef HAVE_CACHE
@@ -83,8 +83,6 @@ int send_file(t_session *session) {
 #ifdef HAVE_COMMAND
 	increment_counter(COUNTER_FILE);
 #endif
-	http_code = 200;
-
 	session->mimetype = get_mimetype(session->file_on_disk, session->config->mimetype);
 
 	handle = -1;
@@ -138,16 +136,17 @@ int send_file(t_session *session) {
 					}
 				}
 			}
-		}
-	}
-	if (invalid_referer) {
-		free(session->file_on_disk);
-		if ((session->file_on_disk = (char*)malloc(strlen(session->host->imgref_replacement) + 4)) != NULL) { // + 3 for ".gz" (gzip encoding)
-			strcpy(session->file_on_disk, session->host->imgref_replacement);
-			session->mimetype = get_mimetype(session->file_on_disk, session->config->mimetype);
-		} else {
-			close(handle);
-			return 500;
+
+			if (invalid_referer) {
+				free(session->file_on_disk);
+				if ((session->file_on_disk = (char*)malloc(strlen(session->host->imgref_replacement) + 4)) != NULL) { // + 3 for ".gz" (gzip encoding)
+					strcpy(session->file_on_disk, session->host->imgref_replacement);
+					session->mimetype = get_mimetype(session->file_on_disk, session->config->mimetype);
+				} else {
+					close(handle);
+					return 500;
+				}
+			}
 		}
 	}
 	
@@ -218,10 +217,14 @@ int send_file(t_session *session) {
 
 	/* Range
 	 */
-	if (*(session->http_version + 7) == '1') {
+	if ((*(session->http_version + 7) == '1') && (session->handling_error == false)) {
 		if ((range = get_headerfield("Range:", session->headerfields)) != NULL) {
 			if (strncmp(range, "bytes=", 6) == 0) {
-				range += 6;
+				if ((range = strdup(range + 6)) == NULL) {
+					close(handle);
+					return 500;
+				}
+			
 				if (split_string(range, &range_begin, &range_end, '-') == 0) {
 
 					if (*range_begin != '\0') {
@@ -229,24 +232,25 @@ int send_file(t_session *session) {
 							if (*range_end != '\0') {
 								if ((send_end = str2int(range_end)) >= 0) {
 									// bytes=XX-XX
-									http_code = 206;
+									session->return_code = 206;
 								}
 							} else {
 								// bytes=XX-
-								http_code = 206;
+								session->return_code = 206;
 							}
 						}
 					} else {
 						if ((send_begin = str2int(range_end)) >= 0) {
 							// bytes=-XX
 							send_begin = file_size - send_begin;
-							http_code = 206;
+							session->return_code = 206;
 						}
 					}
 
-					if (http_code == 206) {
+					if (session->return_code == 206) {
 						if (send_begin >= file_size) {
 							close(handle);
+							free(range);
 							return 416;
 						}
 						if (send_begin < 0) {
@@ -259,6 +263,7 @@ int send_file(t_session *session) {
 							send_size = send_end - send_begin + 1;
 						} else {
 							close(handle);
+							free(range);
 							return 416;
 						}
 					}
@@ -267,25 +272,33 @@ int send_file(t_session *session) {
 					 */
 					if (send_begin > 0) {
 						if (lseek(handle, send_begin, SEEK_SET) == -1) {
-							http_code = 200;
+							session->return_code = 200;
 						}
 					}
 
-					if (http_code == 200) {
+					if (session->return_code == 200) {
 						send_begin = 0;
 						send_end = file_size - 1;
 						send_size = file_size;
 					}
 				}
+				free(range);
 			}
 		}
 	}
 
 	do {
 		retval = -1;
-		if (send_header(session, http_code) == -1) {
+		if (send_header(session, session->return_code) == -1) {
 			break;
 		}
+		if (session->return_code == 401) {
+			if (session->host->auth_method == basic) {
+				send_basic_auth(session);
+			} else {
+				send_digest_auth(session);
+			}
+		}
 
 		value[VALUE_SIZE] = '\0';
 
@@ -305,7 +318,7 @@ int send_file(t_session *session) {
 
 		/* Content-Range
 		 */
-		if (http_code == 206) {
+		if (session->return_code == 206) {
 			if (send_buffer(session, "Content-Range: bytes ", 21) == -1) {
 				break;
 			} else if (snprintf(value, VALUE_SIZE, "%lld-%lld/%lld\r\n", (long long)send_begin, (long long)send_end, file_size) == -1) {
@@ -411,6 +424,10 @@ int execute_cgi(t_session *session) {
 	int retval = 200, result, handle;
 	char *line, *new_line, *contyp_start, *contyp_end, *code;
 	bool in_body = false, send_in_chunks = true, wrap_cgi;
+#ifdef CYGWIN
+	char *old_path, *win32_path;
+#endif
+	t_cgi_result cgi_result;
 	t_connect_to *connect_to;
 	t_cgi_info cgi_info;
 	pid_t cgi_pid = -1;
@@ -504,7 +521,11 @@ int execute_cgi(t_session *session) {
 						return rr_SQL_INJECTION;
 					}
 					session->vars = line;
-					register_tempdata(&(session->tempdata), session->vars, tc_DATA);
+					if (register_tempdata(&(session->tempdata), session->vars, tc_data) == -1) {
+						free(session->vars);
+						session->vars = NULL;
+						return -1;
+					}
 			}
 		}
 
@@ -521,7 +542,11 @@ int execute_cgi(t_session *session) {
 					}
 					session->body = line;
 					session->content_length = result;
-					register_tempdata(&(session->tempdata), session->body, tc_DATA);
+					if (register_tempdata(&(session->tempdata), session->body, tc_data) == -1) {
+						free(session->body);
+						session->body = NULL;
+						return -1;
+					}
 			}
 		}
 
@@ -537,7 +562,11 @@ int execute_cgi(t_session *session) {
 						return rr_SQL_INJECTION;
 					}
 					session->cookie = line;
-					register_tempdata(&(session->tempdata), session->cookie, tc_DATA);
+					if (register_tempdata(&(session->tempdata), session->cookie, tc_data) == -1) {
+						free(session->cookie);
+						session->cookie = NULL;
+						return -1;
+					}
 			}
 		}
 	}
@@ -559,6 +588,21 @@ int execute_cgi(t_session *session) {
 	cgi_info.input_buffer_size = cgi_info.error_buffer_size = CGI_BUFFER_SIZE;
 	cgi_info.input_len = cgi_info.error_len = 0;
 
+#ifdef CYGWIN
+	if ((session->config->platform == windows) && ((session->cgi_type == fastcgi) || (session->cgi_type == script))) {
+		if ((old_path = strdup(session->file_on_disk)) == NULL) {
+			return -1;
+		}
+		if ((win32_path = strdup(cygwin_to_windows(old_path))) == NULL) {
+			free(old_path);
+			return -1;
+		}
+		free(session->file_on_disk);
+		session->file_on_disk = win32_path;
+		free(old_path);
+	}
+#endif
+
 	if (session->cgi_type == fastcgi) {
 		cgi_info.read_header = true;
 		if ((connect_to = select_connect_to(session->fcgi_server, session->ip_address)) == NULL) {
@@ -583,10 +627,10 @@ int execute_cgi(t_session *session) {
 	}
 
 	if ((cgi_info.input_buffer = (char*)malloc(cgi_info.input_buffer_size + 1)) == NULL) {
-		retval = 500;
+		retval = -1;
 	} else if ((cgi_info.error_buffer = (char*)malloc(cgi_info.error_buffer_size + 1)) == NULL) {
 		free(cgi_info.input_buffer);
-		retval = 500;
+		retval = -1;
 	}
 
 	if (retval != 200) {
@@ -605,13 +649,20 @@ int execute_cgi(t_session *session) {
 	cgi_info.timer = session->config->time_for_cgi;
 
 	do {
-		if (session->cgi_type == fastcgi) {
-			result = read_from_fcgi_server(session, &cgi_info);
-		} else {
-			result = read_from_cgi_process(session, &cgi_info);
-		}
+#ifdef DEBUG
+		set_client_status(session, "execute_cgi()::read_from_cgi()");
+#endif
 
-		switch (result) {
+		if (session->cgi_type == fastcgi) {
+			cgi_result = read_from_fcgi_server(session, &cgi_info);
+		} else {
+			cgi_result = read_from_cgi_process(session, &cgi_info);
+		}
+#ifdef DEBUG
+		set_client_status(session, NULL);
+#endif
+
+		switch (cgi_result) {
 			case cgi_ERROR:
 				log_file_error(session, session->file_on_disk, "error while executing CGI");
 				retval = 500;
@@ -695,8 +746,17 @@ int execute_cgi(t_session *session) {
 								if ((retval = str2int(code)) <= 0) {
 									log_file_error(session, session->file_on_disk, "invalid status code received from CGI");
 									retval = 500;
+									break;
+								}
+								session->return_code = retval;
+								if (retval == 200) {
+									if (send_header(session, retval) == -1) {
+										retval = rs_DISCONNECT;
+										break;
+									}
+								} else {
+									break;
 								}
-								break;
 							} else if (send_header(session, session->return_code) == -1) {
 								retval = rs_DISCONNECT;
 								break;
@@ -1218,7 +1278,8 @@ static t_access allow_alter(t_session *session) {
 
 int handle_put_request(t_session *session) {
 	int handle, bytes_written, total_written = 0, result = 204;
-	char *range, *rest;
+	char *range, *value, *rest;
+	bool range_found;
 	long write_begin, write_end, total_size;
 	off_t file_size;
 
@@ -1244,10 +1305,11 @@ int handle_put_request(t_session *session) {
 	}
 
 	range = get_headerfield("Content-Range:", session->headerfields);
+	range_found = (range != NULL);
 
 	if ((handle = open(session->file_on_disk, O_WRONLY)) == -1) {
 		// New file
-		if (range != NULL) {
+		if (range_found) {
 			return 416;
 		}
 		if ((handle = open(session->file_on_disk, O_CREAT | O_WRONLY)) == -1) {
@@ -1273,19 +1335,19 @@ int handle_put_request(t_session *session) {
 		log_file_error(session, session->file_on_disk, "can't change file mode");
 	}
 
-	if (range != NULL) {
+	if (range_found) {
 		if (strncmp(range, "bytes ", 6) != 0) {
 			result = 416;
 		} else {
-			range += 6;
+			range = strdup(range + 6);
 
-			if (split_string(range, &range, &rest, '-') == -1) {
+			if (split_string(range, &value, &rest, '-') == -1) {
 				result = 416;
-			} else if ((write_begin = str2int(range)) == -1) {
+			} else if ((write_begin = str2int(value)) == -1) {
 				result = 416;
-			} else if (split_string(rest, &range, &rest, '/') == -1) {
+			} else if (split_string(rest, &value, &rest, '/') == -1) {
 				result = 416;
-			} else if ((write_end = str2int(range)) == -1) {
+			} else if ((write_end = str2int(value)) == -1) {
 				result = 416;
 			} else if ((total_size = str2int(rest)) == -1) {
 				result = 416;
@@ -1302,11 +1364,13 @@ int handle_put_request(t_session *session) {
 					result = 500;
 				}
 			}
+
+			free(range);
 		}
 	}
 
 	if (result == 204) {
-		if ((file_size != NEW_FILE) && (range == NULL)) {
+		if ((file_size != NEW_FILE) && (range_found == false)) {
 			ftruncate(handle, session->content_length);
 		}
 		while (total_written < session->content_length) {
